%bcond_with shared_libuv

%define major 14
%define minor 16
%define fix 1

%define node_prefix %{_libdir}/%{name}

Name: nodejs%{major}
Version: %{major}.%{minor}.%{fix}
Release: 3
License: MIT
Summary: Node.js JavaScript Runtime
Url: https://www.nodejs.org


BuildRequires: python3
BuildRequires: zlib-devel
BuildRequires: openssl-devel >= 1.1.1
%if %{with shared_libuv}
BuildRequires: libuv-devel >= 1.35.0
Requires: libuv1 >= 1.35.0
%endif

# Previous versions of OpenSSL 1.1 export memcpy/strcpy/etc due to -bexpall,
# but the GA version has a manual export list, so they are not exported.
# Due to order passed to the linker, node 10 used to link to these functions
# explicitly which causes load failures if we don't upgrade in lock-step.
#
# Because nodejs12 is now built against openssl1_1 versions that have ENGINE
# support enabled, need to ensure that a version of OpenSSL built with ENGINE
# support enabled is installed
Requires: libopenssl1_1 >= 1.1.1-6
Requires: python3
Requires: update-alternatives

Requires(post): update-alternatives
Requires(postun): update-alternatives

Conflicts: nodever < 1.0.0

Source0: https://nodejs.org/dist/v%{version}/node-v%{version}.tar.gz

Patch0: nodejs14-fix-vlan-crash.patch

%description
Node.js is a JavaScript runtime built on Chrome V8 JavaScript engine. Node.js
uses an event-driven, non-blocking I/O model that makes it lightweight and
efficient.

%prep
%setup -q -n node-v%{version}

%patch0 -p1

# Fix the wrong dependencies /bin/sh and /usr/bin/python
find deps tools ./configure* -type f | xargs perl -pi -e 's|/usr/bin/python|%{_bindir}/python3|g'

%build

export CC=gcc
export CXX=g++

./configure \
  --prefix=%{node_prefix} \
  --dest-cpu=ppc64 \
  --without-node-snapshot \
  --shared-openssl \
  --shared-zlib \
  %{?with_shared_libuv:--shared-libuv} \
  # end

%make_build
# basic verification
./node -e "console.log(process.versions)"

%check

make test-only

%install

%make_install V=1 PORTABLE=1

# Remove weak libstdc++ symbols exported by node binary from export file
# These are not exported when built with GCC 10 (which we use to build node 16),
# since they are now exported by libstdc++. This won't break add-ons, since
# GCC 6 will statically link these as weak symbols in to the addon shared obj.
sed -i \
  -e '/_ZN9__gnu_cxx12__to_xstringINSt7__cxx1112basic_stringIcSt11char_traitsIcESaIcEEEcEET_PFiPT0_mPKS8_PcEmSB_z/d' \
  -e '/_ZNSt6vectorIcSaIcEE12emplace_backIJcEEEvDpOT_/d' \
  %{buildroot}%{node_prefix}/include/node/node.exp

%post
###############################################################################
# processing after the install
# $1 = 1: install
# $1 = 2+: upgrade
###############################################################################

type=upgrade
[ $1 -eq 1 ] && type=install

if [ $type = 'install' ]
then
  # set up the just installed version of Node.js in update-alternatives
  %{_bindir}/alternatives --install %{_bindir}/node node %{_libdir}/nodejs%{major}/bin/node %{major} \
  --slave %{_bindir}/npm npm %{_libdir}/nodejs%{major}/bin/npm \
  --slave %{_bindir}/npx npx %{_libdir}/nodejs%{major}/bin/npx
fi


%postun
###############################################################################
# processing after the uninstall
# $1 = 0: uninstall
# $1 = 1: upgrade
###############################################################################

type=uninstall
[ $1 -eq 1 ] && type=upgrade

# After uninstallation, need to check if it was truly an uninstallation, and if
# so, remove the the update-alternatives entry for this version of node
if [ $type = 'uninstall' ]
then
  %{_bindir}/alternatives --remove node %{_libdir}/nodejs%{major}/bin/node
fi


%files
# Set the default owner to QSYS and group to *NONE
%defattr(-, qsys, *none)

%{node_prefix}

%changelog
* Tue May 25 2021 Kevin Adler <kadler@us.ibm.com> - 14.16.1
- Remove libstdc++ symbols from node.exp

* Thu Apr 08 2021 Kevin Adler <kadler@us.ibm.com> - 14.16.1
- Fix crash on systems with VLAN interface names > 10 characters

* Wed Apr 7 2021 Xu Meng <mengxumx@cn.ibm.com> - 14.16.1
- update to v14.16.1 to address three vulnerabilities -
- CVE-2021-3450
- CVE-2021-3449
- CVE-2020-7774

* Thu Feb 25 2021 Xu Meng <mengxumx@cn.ibm.com> - 14.16.0
- update to v14.16.0 to address three vulnerabilities -
- CVE-2021-22883
- CVE-2021-22884
- CVE-2021-23840

* Tue Jan 05 2021 Xu Meng <mengxumx@cn.ibm.com> - 14.15.4
- update to v14.15.4 to address three vulnerabilities -
- CVE-2020-8265
- CVE-2020-8287
- CVE-2020-1971

* Tue Nov 17 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.15.1
- Update to v14.15.1 to address a vulnerability -
- CVE-2020-8277

* Mon Sep 21 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.11.0
- Update to v14.11.0 to address three vulnerabilities -
- CVE-2020-8251
- CVE-2020-8201

* Wed Jul 22 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.6.0
- Update to v14.6.0

* Wed Jul 8 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.5.0
- Update to v14.5.0
- V8 engine is updated to version 8.3
- Backport V8 fix for the API pthread_getattr_np() on 7.2

* Wed Jun 3 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.4.0
- update to v14.4.0 to address three vulnerabilities -
- CVE-2020-8172
- CVE-2020-11080
- CVE-2020-8174

* Mon Jun 1 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.3.0
- Update to v14.3.0

* Tue May 19 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.2.0
- Update to v14.2.0

* Wed Apr 22 2020 Xu Meng <mengxumx@cn.ibm.com> - 14.0.0
- Update to v14.0.0
- Switch to using update alternatives
- Conflict with versions of nodever < 1.0.0

* Thu Feb 20 2020 Xu Meng <mengxumx@cn.ibm.com>
- Initial nightly build for verification

